@@ -39,13 +39,9 @@
    chdir ("..");
    define('SM_PATH','../');
 
+
 // include compatibility plugin
-if (defined('SM_PATH'))
    include_once(SM_PATH . 'plugins/compatibility/functions.php');
-else if (file_exists('../plugins/compatibility/functions.php'))
-   include_once('../plugins/compatibility/functions.php');
-else if (file_exists('./plugins/compatibility/functions.php'))
-   include_once('./plugins/compatibility/functions.php');
 
 if (compatibility_check_sm_version(1, 3)) {
    if (!isset($config_php))
@@ -60,39 +56,30 @@ if (compatibility_check_sm_version(1, 3)) {
    include_once (SM_PATH . "include/load_prefs.php");
 } else {
    //session_start(); 
-   include_once("../src/validate.php");
+   include_once(SM_PATH . "src/validate.php");
    if (!isset($config_php))
-      include_once ("../config/config.php");
+      include_once (SM_PATH . "config/config.php");
    if (!isset($strings_php))
-      include_once ("../functions/strings.php");
+      include_once (SM_PATH . "functions/strings.php");
    if (!isset($page_header))
-      include_once ("../functions/page_header.php");
+      include_once (SM_PATH . "functions/page_header.php");
    if (!isset($imap_php))
-      include_once ("../functions/imap.php");
-   include_once ("../src/load_prefs.php");
+      include_once (SM_PATH . "functions/imap.php");
+   include_once (SM_PATH . "src/load_prefs.php");
 }
 
-function do_international() {
-   bindtextdomain('change_ldappass', '../plugins/change_ldappass/locale');
-   textdomain('change_ldappass');
-}
+bindtextdomain('change_ldappass', SM_PATH . 'plugins/change_ldappass/locale');
+textdomain('change_ldappass');
+if (isset($_POST['plugin_change_ldappass'])) $Messages = change_ldappass_check();
+textdomain('squirrelmail');
+displayPageHeader($color, "None");
+textdomain('change_ldappass');
 
-function undo_international() {
-    bindtextdomain('squirrelmail', SM_PATH . 'locale');
-    textdomain('squirrelmail');
-}
-
-   if (isset($_POST['plugin_change_ldappass'])) $Messages = change_ldappass_check();
-   displayPageHeader($color, "None");
-
-do_international();
-?>
-
-<br>
-<table width=95% align=center cellpadding=2 cellspacing=2 border=0>
-<tr><td bgcolor="<?php echo $color[0] ?>">
-   <center><b><?php echo _("Change Password") ?></b></center>
-</td><?php
+echo '<br>
+	<table width=95% align=center cellpadding=2 cellspacing=2 border=0>
+	<tr><td bgcolor="' . $color[0] . '">
+	   <center><b>' . _("Change Password") . '</b></center>
+	</td>' ;
 
 if (isset($Messages) && count($Messages)) {
     echo "<tr><td>\n";
@@ -102,30 +89,33 @@ if (isset($Messages) && count($Messages)) {
     echo "</td></tr>\n";
 }
 
-?><tr><td>
-    <form method=post action="<?php print $_SERVER['PHP_SELF'] ?>">
-    <table>
-      <tr>
-        <th align=right><?php echo _("Old Password:")?></th>
-        <td><input type=password name=cp_oldpass value=""  size=20></td>
-      </tr>
-      <tr>
-        <th align=right><?php echo _("New Password:")?></th>
-        <td><input type=password name=cp_newpass value="" size=20></td>
-      </tr>
-      <tr>
-        <th align=right><?php echo _("Verify New Password:")?></th>
-        <td><input type=password name=cp_verify value="" size=20></td>
-      </tr>
-      <tr>
-        <td align=center colspan=2><input type=submit value="<?php echo _("Change Password") ?>" name="plugin_change_ldappass"></td>
-      </tr>
-    </table>
-</td></tr>
-</tr></table>
-</body></html>
-<?php undo_international(); ?>
-<?php
+echo
+  '<tr><td>
+      <form method=post action="' . $_SERVER['PHP_SELF'] . '">
+      <table>
+        <tr>
+          <th align=right>'. _("Old Password:") .'</th>
+          <td><input type=password name=cp_oldpass value=""  size=20></td>
+        </tr>
+        <tr>
+          <th align=right>' . _("New Password:") .'</th>
+          <td><input type=password name=cp_newpass value="" size=20></td>
+        </tr>
+        <tr>
+          <th align=right>' .  _("Verify New Password:") . '</th>
+          <td><input type=password name=cp_verify value="" size=20></td>
+        </tr>
+        <tr>
+          <td align=center colspan=2><input type=submit value="' . _("Change Password") . '" name="plugin_change_ldappass"></td>
+        </tr>
+      </table>
+  </td></tr>
+  </tr></table>
+  </body></html>' ;
+
+textdomain('squirrelmail');
+
+// FUNCTIONS
 
 function change_ldappass_check($debug = 0) {
 	$cp_oldpass = $_POST['cp_oldpass'];
@@ -135,27 +125,24 @@ function change_ldappass_check($debug = 0) {
 	$username = $_SESSION['username'];
 
 	$Messages = array();
-	do_international();
 	if ($cp_oldpass == "")	array_push($Messages, _("You must type in your old password."));
 	if ($cp_newpass == "") array_push($Messages, _("You must type in a new password."));
 	if ($cp_verify == "") array_push($Messages, _("You must also type in your new password in the verify box."));
 	if ($cp_newpass != "" && $cp_verify != $cp_newpass) array_push($Messages, _("Your new password doesn't match the verify password."));
-	if (!preg_match("/^[A-Za-z0-9\\%\^\*\(\)\-\_\+\=\[\]\{\}\:\@\#\~\,\.\? ]+$/",$cp_newpass)) array_push($Messages,'Passwords can only contain the following characters:<br>A-Z, a-z, 0-9, %^*()-_+=[]{}:@#~,.?');
-        undo_international();
+	if (!preg_match("/^[A-Za-z0-9\\%\^\*\(\)\-\_\+\=\[\]\{\}\:\@\#\~\,\.\? ]+$/",$cp_newpass)) array_push($Messages, _('Passwords can only contain the following characters:')."<br>A-Z, a-z, 0-9, %^*()-_+=[]{}:@#~,.?");
 	if (count($Messages)) return $Messages;
 	return change_ldappass_go($debug);
 }
 
 
 function change_ldappass_go($debug) {
-		$username = $_SESSION['username'];
-		$base_uri = $_SESSION['base_uri'];
-		$cp_oldpass = $_POST['cp_oldpass'];
-		$cp_newpass = $_POST['cp_newpass'];
+    $username = $_SESSION['username'];
+    $base_uri = $_SESSION['base_uri'];
+    $cp_oldpass = $_POST['cp_oldpass'];
+    $cp_newpass = $_POST['cp_newpass'];
 
-		include_once("../plugins/change_ldappass/config.php");
+    include_once("../plugins/change_ldappass/config.php");
     $Messages = array();
-    do_international(); 
     if ($debug) array_push($Messages, _("Connecting to LDAP Server"));
 
  		$ds=ldap_connect($ldap_server);
@@ -163,17 +150,18 @@ function change_ldappass_go($debug) {
 			array_push($Messages, _("Can't connect to Directory Server, please try later!"));
 	 		return $Messages;
  		}
-		if ($ldap_bind_as_manager) {
-			if (! @ldap_bind($ds,$ldap_manager_dn,$ldap_manager_pw)) {
-				array_push($Messages, _("Could not bind to the LDAP server as $ldap_manager_dn. Is the password correct?"));
-				return $Messages;
-			} else {
-				if ($debug) array_push($Messages, _("Bind to database as $ldap_manager_dn successful."));
-			}				
+		// first bind and try to determine the correct dn for the user with uid=$username
+		if (! @ldap_bind($ds,"$query_dn","$query_pw")) {
+			array_push($Messages, _("LDAP bind failed."));
+			if ($debug) array_push($Messages, "BIND-DN: $ldap_manager_dn");
+			return $Messages;
 		} else {
-			// first bind anonymously and try to determine the correct dn for the user with uid=$username
-			$r=ldap_bind($ds,"","");
+			if ($debug) {
+				array_push($Messages, _("LDAP bind successful."));
+				array_push($Messages, "BIND-DN: $query_dn");
+			}
 		}
+
 		$sr=ldap_search($ds,$ldap_base_dn,"($ldap_user_field=$username)",array("dn")); //search for uid
 		if (ldap_count_entries($ds,$sr)>1) {
 			array_push($Messages, _("Duplicate login entries detected, cannot change password!"));
@@ -187,15 +175,36 @@ function change_ldappass_go($debug) {
 		if ($debug) array_push($Messages,ldap_debug_print_array($info));
 		$dn=$info[0]["dn"]; //finally get the full users dn
 		
-		// now rebind the the database as user or as root if required
-		if (! $ldap_bind_as_manager) {
+		// now rebind to the database as user to verify password.
+		if (@! "$no_bind_as_user") {
 			if (! @ldap_bind($ds,$dn,$cp_oldpass)) { //if we can't bind as the user then the old passwd must be wrong
 				array_push($Messages, _("Your old password is not correct."));
+				if ($debug) {
+					array_push($Messages, _("LDAP bind failed."));	
+					array_push($Messages, "BIND-DN: $dn");
+				}
 				return $Messages;
 			} else {
-				if ($debug) array_push($Messages, _("Bind to database as $dn successful."));	
+				if ($debug) {
+					array_push($Messages, _("LDAP bind successful."));	
+					array_push($Messages, "BIND-DN: $dn");
+				}
 			}
 		}
+
+		// Now, if needed, we rebind as the manager so we can read passwords and make changes.
+		if ($ldap_bind_as_manager) {
+			if (! @ldap_bind($ds,$ldap_manager_dn,$ldap_manager_pw)) {
+				array_push($Messages, _("LDAP bind failed."));
+				if ($debug) array_push($Messages, "BIND-DN: $ldap_manager_dn");
+				return $Messages;
+			} else {
+				if ($debug) { 
+					array_push($Messages, _("LDAP bind successful."));
+					array_push($Messages, "BIND-DN: $ldap_manager_dn");
+				}
+			}				
+		}
 		$sr=ldap_search($ds,$ldap_base_dn,"($ldap_user_field=$username)"); //check the db again, this time so we get the password field returned
 		$info = ldap_get_entries($ds, $sr);
 		if ($debug) array_push($Messages,ldap_debug_print_array($info));
@@ -228,25 +237,25 @@ function change_ldappass_go($debug) {
 				$stype="DES";		 						//sensible default if not detected
 				if ($pl>=34 and substr($lpass,0,3)=="$1$") $stype="MD5"; 
 				if ($pl>=34 and substr($lpass,0,3)=="$2$") $stype="BLOWFISH";	
-				if ($debug) array_push($Messages, _("Password type is {crypt}, sub-type $stype"));
+				if ($debug) array_push($Messages, _("Password type is") . " {crypt}, sub-type $stype");
 				$cpass=ldap_crypt_passwd($cp_oldpass,$lpass,$stype); //crypt up our old password so we can check it again
 				break;
 			case "{md5":
-				if ($debug) array_push($Messages, _("Password type is {MD5}"));
+				if ($debug) array_push($Messages, _("Password type is") . " {MD5}" );
 				$cpass=ldap_md5_passwd($cp_oldpass);
 				break;
 			case "{sha":
-				if ($debug) array_push($Messages, _("Password type is {SHA}"));
+				if ($debug) array_push($Messages, _("Password type is") . " {SHA}");
 				$cpass=ldap_sha_passwd($cp_oldpass);
 				break;
 			case "{smd5":
-				if ($debug) array_push($Messages, _("Password type is {SMD5}"));
+				if ($debug) array_push($Messages, _("Password type is") . " {SMD5}");
 				$hash = base64_decode($lpass) ;
 				$salt = substr($hash, 16);
 				$cpass = base64_encode(pack("H*", md5($cp_oldpass . $salt)).$salt);
 				break;
 			case "{ssha":
-				if ($debug) array_push($Messages, _("Password type is {SSHA}"));
+				if ($debug) array_push($Messages, _("Password type is") . " {SSHA}");
 				$hash = base64_decode($lpass) ;
 				$salt = substr($hash, 20);
 				$cpass=ldap_ssha_passwd($cp_oldpass,$salt);
@@ -259,7 +268,7 @@ function change_ldappass_go($debug) {
 		//now check again the stored password against the encrypted version of the supplied old password
 		if ($lpass != $cpass) {
 			array_push($Messages, _("Your old password is not correct."));
-			if ($debug) array_push($Messages, _("Stored Password: $lpass  Old Password: $cpass"));
+			if ($debug) array_push($Messages, _("Stored Password:") . " $lpass " . _("Old Password:") . " $cpass");
 			return $Messages;
 		}
 		//Make sure the new passwd generation uses the encryption method of the previous password
@@ -279,24 +288,23 @@ function change_ldappass_go($debug) {
 			case "{ssha":
 				@ $newpass="{SSHA}".ldap_ssha_passwd($cp_newpass);
 				break;
-				
 			// more password types should go here ;-) and the functions to drive them below.
-	    default:                        // Use plain text password
-      	$newpass=$cp_newpass;
-        break;
+			default:                        // Use plain text password
+      				$newpass=$cp_newpass;
+       				 break;
 		}
-		if ($debug) array_push($Messages, _("New Password: $newpass"));
+		if ($debug) array_push($Messages, _("New Password:") . " $newpass");
 		$newinfo=array();
 		$newinfo[$ldap_password_field][0]=$newpass;
 		if ($change_ldapsmb) {
-			$exe = "$mkntpwd " . escapeshellarg($newpass) . " 2>&1" ;
+			$exe = "$mkntpwd " . escapeshellarg($cp_newpass) . " 2>&1" ;
 			if ($debug) array_push($Messages,$exe);
 			$ntString = exec ($exe, $retarray, $retval);
 			if ($debug) $Messages = array_merge($Messages,$retarray);
 			if ( $retval == "0" && preg_match("/^[0-9A-F]+:[0-9A-F]+$/",$ntString )) {
 				list($lmPassword, $ntPassword) = explode (":", $ntString);
-				$newinfo["ntPassword"] = $ntPassword;
-				$newinfo["lmPassword"] = $lmPassword;
+				$newinfo[$ldapsmb_ntpassword] = $ntPassword;
+				$newinfo[$ldapsmb_lmpassword] = $lmPassword;
 			} else {  //smbpasswd change failed so we must re sync the ldap password back to its original
 				array_push($Messages, _("SMB Password change was not successful, so LDAP not changed!"));
 				return $Messages;
@@ -305,12 +313,19 @@ function change_ldappass_go($debug) {
 		if (@ldap_modify($ds,$dn,$newinfo)) {
 			$smb=0;
 			if ($change_smb) {
-				$exe="echo " . escapeshellarg($cp_oldpass)  
-				       .  " |$smb_passwd -U " . escapeshellarg($username) 
-				       . " -s " . escapeshellarg($cp_newpass) . " 2>&1" ;
+				if ($smb_host) {
+					$exe="echo " . escapeshellarg($cp_oldpass)  
+					       .  " |$smb_passwd -r " . escapeshellarg($smb_host)
+					       . " -U " . escapeshellarg($username) 
+					       . " -s " . escapeshellarg($cp_newpass) . " 2>&1" ;
+				} else {
+					$exe="echo " . escapeshellarg($cp_oldpass)  
+					       .  " |$smb_passwd -U " . escapeshellarg($username) 
+					       . " -s " . escapeshellarg($cp_newpass) . " 2>&1" ;
+				}
 				if ($debug) array_push($Messages,$exe);
 				$r=exec($exe,$s);
-				if ($r==_("Password changed for user $username")) $smb=1 ;
+				if ($r==_("Password changed for user") . " $username") $smb=1 ;
 			} else {
 				$smb=1;
 			}
@@ -341,7 +356,6 @@ function change_ldappass_go($debug) {
 			return $Messages;
 		}
 		@ldap_close($ds);
-		undo_international();
 }
 
 // Generate an unsalted SHA1 pw. This should work withNetscape messaging / directory server 4+ 
@@ -397,7 +411,7 @@ function ldap_generate_salt($stype) {
   $salt="";	//generate a salt using characters [A-Z][a-z][0-9]./
   $chars="./0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
 	for ($i=0;$i<16;$i++) {
-		$salt.=$chars[mt_rand(0,strlen($chars))];
+		@ $salt.=$chars[mt_rand(0,strlen($chars))];
 	}
 	//return salts longer than we need but these will be trimmed in the ldap_crypt_passwd function
 	if ($stype=="MD5") return "$1$".$salt;
